;;; FG42 --- The mighty editor for the emacsians -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani <lxsameer@gnu.org>
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 4.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;; Code:
(eval-when-compile
  (require 'fpkg))

(require 'fg42/modeline/units)


(defun fg42/--bottom-right (info)
  "Keep the modeline at bottom right by using the data from INFO."
  (cons -1 -1))

(defun fg42/--bottom-right-padded (info)
  "Keep the modeline at bottom right by using the data from INFO."
  (cons -70 -1))


(defun fg42/adjust-modeline (view)
  "Adjust the VIEW after parent frame resize."
  (noether-show view))


(noether-defview fg42/modeline
  "A simple and minimalist mode-line like status bar"
  :managed? t
  :binding (kbd "C-c 0")
  :buffer "*modeline*"
  :visible? t
  :timeout 0
  :on-parent-resize #'fg42/adjust-modeline
  :frame
  (list
   :right-fringe 5
   :poshandler #'fg42/--bottom-right
   :border-width 0
   :font (format "%s %s" (car fg42/font) (- (cadr fg42/font) 1))
   :border-color "#bd93f9")

  :units
  (list
   (buffer-name-unit
    :label (format "%s " (nerd-icons-codicon "nf-cod-layers"))
    :len 20)
   (projectile-project-unit
    :label (format "%s " (nerd-icons-octicon "nf-oct-project"))
    :len 20)
   (git-branch-unit
    :label (format "%s " (nerd-icons-devicon "nf-dev-git_branch"))
    :len 20)
   (fg42/mode-icon)
   (line-unit :label (format "%s " (nerd-icons-codicon "nf-cod-location")))
   (time-unit :label (format " %s " (nerd-icons-mdicon "nf-md-clock_time_three")))))


(noether-defview fg42/minimal-exwm
  "A super simple bar containing the line number and column number that
Appears on the center of the current window."
  :managed? t
  :buffer "*exwm-status*"
  :binding (kbd "C-c 1")
  :separator " | "

  :timeout 10
  :frame
  (list
   :poshandler #'fg42/--bottom-right-padded
   :border-width 0
   :border-color "#bd93f9")

  :units
  (list
   (fg42/exwm-input-mode-unit :label (format "%s " (nerd-icons-faicon "nf-fa-linux")))
   (buffer-name-unit
    :label (format "%s " (nerd-icons-codicon "nf-cod-layers"))
    :len 30)
   
   (time-unit :label (format "%s " (nerd-icons-mdicon "nf-md-clock_time_three")))))


(provide 'fg42/modeline/views)
;;; views.el ends here
