;;; FG42 --- The mighty editor for the emacsians -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 4.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;; Code:

(defmacro fg42/config-get (key)
  "Return the value for KEY or raise an error."
  (let ((sym (intern (format "fg42/config/%s" key))))
    (if (boundp sym)
        `,sym
      `(error "Can't find config '%s'" ,key))))

(defmacro fg42/config-get-or (key &optional default)
  "Return the value for KEY or DEFAULT."
  (let ((sym (intern (format "fg42/config/%s" key))))
    (if (boundp sym)
        `,sym
      `,default)))


(defmacro with-config (name &rest body)
  "Run the BODY only if the config NAME is set to t."
  (declare (indent defun))
  `(when (string= (format "%s" (fg42/config-get-or ,name "")) "t")
     ,@body))


(provide 'fg42/config)
;;; config.el ends here
